--[[--
json_string =  -- JavaScript code representing the given datum (with quotes, if needed)
encode.json(
  obj          -- true, false, nil or a number or string
)

This function encodes any native datatype or atom in JavaScript object notation (JSON).

--]]--

function encode.json(obj)
  if obj == nil then
    return "null";
  elseif atom.has_type(obj, atom.boolean) then
    return tostring(obj)
  elseif atom.has_type(obj, atom.number) then
    return tostring(obj)
  else
    return
      "'" ..
      string.gsub(atom.dump(obj), ".",
        function (char)
          if char == "\r" then return "\\r"  end
          if char == "\n" then return "\\n"  end
          if char == "\\" then return "\\\\" end
          if char == "'"  then return "\\'"  end
          if char == "/"  then return "\\/"  end  -- allowed according to RFC4627, needed for </script>
          local byte = string.byte(char)
          if byte < 32 then return string.format("\\u%04x", byte) end
        end
      ) ..
      "'"
  end
end
